@echo off
REM ############################################################
REM # APPLICATION: 	HealtheVetDesktop (Care Management)
REM # VERSION:		1.8.0
REM # AUTHOR:		Parminder Singh
REM # DESCRIPTION:	This batch file is used to launch the 
REM #				HealtheVet application and replacement of 
REM #				the Delphi launcher code.
REM #
REM ############################################################
TITLE HeatheVetDesktop_1.8.0
REM
set APP_STARTUP_DIR=%~dp0
REM
set LOG_FILE_NAME=cm_session.log
set LOG_FILE_PATH=%APP_STARTUP_DIR%%LOG_FILE_NAME%
REM
set CONFIG_FILE_NAME=config.xml
set CONFIG_FILE_PATH=%APP_STARTUP_DIR%%CONFIG_FILE_NAME%
REM
set PLUGINS_FOLDER=plugins\gov.va.med.hds.cd.mum_1.8.0
set BOOT_JAR_FILE_NAME=boot.jar
set BOOT_JAR_FILE_PATH=%APP_STARTUP_DIR%%PLUGINS_FOLDER%\%BOOT_JAR_FILE_NAME%
REM
if Exist "%LOG_FILE_PATH%" (
	del "%LOG_FILE_PATH%"
)
REM
echo.========================================================================== >> "%LOG_FILE_NAME%"
echo.[%date%%time%]: Started application / created logger >> "%LOG_FILE_NAME%"
echo.========================================================================== >> "%LOG_FILE_NAME%"
REM
echo.[%date%%time%]: Startup dir=%APP_STARTUP_DIR% >> "%LOG_FILE_NAME%"
echo.[%date%%time%]: Log file path=%LOG_FILE_PATH% >> "%LOG_FILE_NAME%"
echo.[%date%%time%]: Config file path=%CONFIG_FILE_PATH% >> "%LOG_FILE_NAME%"
echo.[%date%%time%]: Boot Jar file path=%BOOT_JAR_FILE_PATH% >> "%LOG_FILE_NAME%"
echo.========================================================================== >> "%LOG_FILE_NAME%"
REM
REM
REM -- Resolve location of 32 bit Java runtime environment
REM
set KeyName=HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\JavaSoft\Java Runtime Environment
echo.[%date%%time%]: Resolve location of 32 bit Java runtime environment using "%KeyName%" >> "%LOG_FILE_NAME%"
REM
set Cmd=reg query "%KeyName%" /s
for /f "tokens=2*" %%i in ('%Cmd% ^| find "JavaHome"') do set JRE_HOME=%%j
REM
echo.[%date%%time%]: JRE path: "%JRE_HOME%" >> "%LOG_FILE_NAME%"
REM
REM -- Resolve location of 32 bit Java JDK environment
REM
set KeyName=HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\JavaSoft\Java Development Kit
echo.[%date%%time%]: Resolve location of 32 bit Java JDK environment using "%KeyName%" >> "%LOG_FILE_NAME%"
REM
set Cmd=reg query "%KeyName%" /s
for /f "tokens=2*" %%i in ('%Cmd% ^| find "JavaHome"') do set JDK_HOME=%%j
REM
echo.[%date%%time%]: JDK path: "%JDK_HOME%" >> "%LOG_FILE_NAME%"
REM
echo.========================================================================== >> "%LOG_FILE_NAME%"
REM -- Check if we located either JRE or JDK exists
REM
if "%JRE_HOME%%JDK_HOME%"=="" ( 
	REM echo Failed to locate any installed Java environments, please install a Java Runtime Environment
	echo.[%date%%time%]: Failed to locate any installed Java environments, please install a Java Runtime Environment >> "%LOG_FILE_NAME%"
)
REM
REM -- If a Java runtime environment is located then set both JAVA_HOME and JAVA_JRE system environment variables
REM
if not "%JRE_HOME%"=="" (
	echo.[%date%%time%]: Setting both JAVA_HOME and JAVA_JRE system environment variables >> "%LOG_FILE_NAME%"
	setx JRE_HOME "%JRE_HOME%"
	if "%JDK_HOME%"=="" ( 
		setx JAVA_HOME "%JRE_HOME%" 
	)
	echo.[%date%%time%]: Done setting both JAVA_HOME and JAVA_JRE system environment variables >> "%LOG_FILE_NAME%"
)
REM
REM
REM -- If a Java JDK environment is located then set JAVA_HOME system environment variables
REM 
if not "%JDK_HOME%"=="" (
	echo.[%date%%time%]: Java JDK environment is located, setting JAVA_HOME system environment variables >> "%LOG_FILE_NAME%"
	setx JAVA_HOME "%JDK_HOME%"
	if "%JRE_HOME%"=="" ( 
		setx JRE_HOME "%JDK_HOME%\jre" 
	)
	echo.[%date%%time%]: Done setting JAVA_HOME system environment variables >> "%LOG_FILE_NAME%"
)
REM
echo.========================================================================== >> "%LOG_FILE_NAME%"
REM
REM -- Append JRE_HOME to the beginning of System PATH
REM
echo.[%date%%time%]: Appending JRE_HOME to the beginning of System PATH >> "%LOG_FILE_NAME%"
REM
set NEWPATH=%JRE_HOME%;%JRE_HOME%\bin;%JRE_HOME%\lib;%PATH%
REM
REM -- Remove double quotes in System PATH variable to prevent error when setting the System PATH with the new value
REM
set NEWPATH_NOQ=%NEWPATH:"=%
REM
REM -- Add semicolon at the end if System PATH if it ends with a backslash to prevent error when setting the System PATH
REM
set LASTCHAR=%NEWPATH_NOQ:~-1%
set backslash=\
if "%LASTCHAR%"=="%backslash%" set NEWPATH_NOQ=%NEWPATH_NOQ%;
REM
REM
REM -- Set the System PATH with the new value
REM
echo.[%date%%time%]: Setting the System PATH with the new value ("%NEWPATH_NOQ%") >> "%LOG_FILE_NAME%"
REM
"C:\Windows\system32\setx.exe" PATH "%NEWPATH_NOQ%" /M
REM
echo.========================================================================== >> "%LOG_FILE_NAME%"
REM
echo.[%date%%time%]: Start application using javaw command... >> "%LOG_FILE_NAME%"
REM
echo.[%date%%time%]: Full command: "%JRE_HOME%\bin\javaw.exe" -D32 -jar %BOOT_JAR_FILE_PATH% %CONFIG_FILE_PATH% >> "%LOG_FILE_NAME%"
REM
start "" "%JRE_HOME%\bin\javaw.exe" -D32 -jar "%BOOT_JAR_FILE_PATH%" "%CONFIG_FILE_PATH%"
REM
echo.[%date%%time%]: Application started... >> "%LOG_FILE_NAME%"
REM
echo.[%date%%time%]: Terminating and closing the command line window... >> "%LOG_FILE_NAME%"
exit
REM pause
REM @echo on